/* packet-hip.h
 * Definitions for packet disassembly structures and routines used both by HIP.
 *
 * $Id: packet-hip.h 25632 2008-07-01 10:59:26Z sklvarjo $
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __PACKET_HIP_H__
#define __PACKET_HIP_H__

/* This minimal structure used to get at the Type field*/
struct newhip {
        guint16 nextpluslen;        /* Next header, plus length */
        guint8 hiptype;        /* Type (what we are after) */
        guint8 hipreserved;        /* Reserved (what we are after) */
        /*variable size*/               /* (IV and) Payload data */
};
 
/* 128-bit Host Identity Tag */
#define HIT_BITSIZE 128
typedef unsigned char hip_hit [HIT_BITSIZE/8];

#define HI_ALG_DSA 3
#define HI_ALG_RSA 5

/* HIP packet types */
typedef enum {
	HIP_I1=1,
	HIP_R1,
	HIP_I2,
	HIP_R2,
	CER,	/* 5 - removed from draft-ietf-hip-base-03 */
	BOS=11,	/* 11 - removed from draft-ietf-hip-base-01 */
	UPDATE=16, /* 16 */
	NOTIFY=17, /* 17 */
	CLOSE=18,  /* 18 */
	CLOSE_ACK=19, /* 19 */
} HIP_PACKETS;

/* HIP TLV parameters */
/* 03/2006 draft-ietf-hip-base-05, -esp-02, -mm-03, -registration-01, -rvs-04 */
#define PARAM_ESP_INFO			65
#define PARAM_R1_COUNTER		128
#define PARAM_LOCATOR			193
#define PARAM_PUZZLE			257
#define PARAM_SOLUTION			321
#define PARAM_SOLUTION_M		322
#define PARAM_SEQ			385
#define PARAM_ACK			449
#define PARAM_DIFFIE_HELLMAN		513
#define PARAM_HIP_TRANSFORM		577
#define PARAM_ENCRYPTED			641
#define PARAM_HOST_ID			705
#define PARAM_CERT			768
#define PARAM_NOTIFY			832
#define PARAM_ECHO_REQUEST		897
#define PARAM_REG_INFO			930
#define PARAM_REG_REQUEST		932
#define PARAM_REG_RESPONSE		934
#define PARAM_REG_FAILED		936
#define PARAM_ECHO_RESPONSE		961
#define PARAM_ECHO_RESPONSE_M		962
#define PARAM_ESP_TRANSFORM		4095
#define PARAM_TRANSFORM_LOW		2048 /* defines range for transforms */
#define PARAM_TRANSFORM_HIGH		4095
#define PARAM_HMAC			61505
#define PARAM_HMAC_2			61569
#define PARAM_HIP_SIGNATURE_2		61633
#define PARAM_HIP_SIGNATURE		61697
#define PARAM_ECHO_REQUEST_NOSIG	63661
#define PARAM_ECHO_RESPONSE_NOSIG	63425
#define PARAM_ECHO_REQUEST_M		65332
#define PARAM_PUZZLE_M			65334
#define PARAM_FROM			65498
#define PARAM_RVS_HMAC			65500
#define PARAM_VIA_RVS			65502
#define PARAM_CRITICAL_BIT		0x0001

#define HIP_CONTROL_C_MASK	0x0002
#define HIP_CONTROL_A_MASK	0x0001

#define HI_HDR_FLAGS_MASK	0xFFFF0000
#define HI_HDR_PROTO_MASK	0x0000FF00
#define HI_HDR_ALG_MASK		0x000000FF

const value_string hi_hdr_flags_vals[] = {
	{ 0x0200, "key is associated with a user" },
	{ 0x0201, "zone key" },
	{ 0x0202, "key is associated with non-zone entity" },
	{ 0x0, "Other" },
};

const value_string hi_hdr_proto_vals[] = {
	{ 0x01, "key is used for TLS" },
	{ 0x02, "key is used for email" },
	{ 0x03, "key is used for DNS security" },
	{ 0x04, "key is used for Oakley/IPSEC" },
	{ 0xFF, "key is valid for any protocol" },
	{ 0x0, NULL },
};

const value_string hi_hdr_alg_vals[] = {
	{ 0x00, "reserved" },
	{ 0x01, "RSA/MD5" },
	{ 0x02, "Diffie-Hellman" },
	{ 0x03, "DSA" },
	{ 0x04, "elliptic curve crypto" },
	{ 0x05, "RSA" },
	{ 0xFF, "reserved" },
};

const value_string notify_vals[] = {
	{ 1, "Unsupported critical parameter type" },
	{ 7, "Invalid syntax" },
	{ 14, "No Diffie-Hellman proposal chosen" },
	{ 15, "Invalid Diffie-Hellman chosen" },
	{ 16, "No HIP proposal chosen" },
	{ 17, "Invalid HIP transform chosen" },
	{ 18, "No ESP proposal chosen" },
	{ 19, "Invalid ESP transform chosen" },
	{ 24, "Authentication failed" },
	{ 26, "Checksum failed" },
	{ 28, "HMAC failed" },
	{ 32, "Encryption failed" },
	{ 40, "Invalid HIT" },
	{ 42, "Blocked by policy" },
	{ 44, "Server busy please retry" },
	{ 46, "I2 acknowledgement" },
	{ 0x0, NULL },
};

typedef struct _hiphdr {
    guint8 proto;        /* payload protocol            */
    guint8 hdr_len;      /* header length               */
    guint8 packet_type;  /* packet type                 */
    guint8 res:4,version:4; /* version, reserved        */
    guint16 checksum;    /* checksum                    */
    guint16 control;     /* control                     */
    hip_hit hit_sndr;    /* Sender's Host Identity Tag  */
    hip_hit hit_rcvr;    /* Receiver's Host Identity Tag*/
    /* HIP parameters ...  */
} hiphdr;

typedef struct _tlv_head
{
    guint16 type;
    guint16 length;
} tlv_head;


/* functions */
char * hip_param(int n);
char *dh_group_id_label(int groupid);
char *transform_id_label(int transform);
char *sig_alg_label(int alg);
int dissect_hip_tlv(tvbuff_t *tvb, int offset, proto_item *ti, int type, int tlv_len);

static dissector_handle_t data_handle;

#endif /* packet-hip.h */
